##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for zope.app.server.accesslog.
"""
import logging
import unittest

from ZConfig.components.logger import loghandler
from ZConfig.components.logger.tests.support import LoggingTestHelper


class LoggingTestBase(LoggingTestHelper, unittest.TestCase):
    pass


class TestAccessLogging(LoggingTestBase):

    name = "accesslog"

    _schematext = """
      <schema>
        <import package='zope.app.server' file='accesslog.xml'/>
        <section type='accesslog' name='*' attribute='accesslog'/>
      </schema>
    """

    def test_config_without_logger(self):
        conf = self.get_config("")
        self.assertIsNone(conf.accesslog)

    def test_config_without_handlers(self):
        logger = self.check_simple_logger("<accesslog/>")
        # Make sure there's a NullHandler, since a warning gets
        # printed if there are no handlers:
        self.assertEqual(len(logger.handlers), 1)
        self.assertIsInstance(logger.handlers[0], loghandler.NullHandler)

    def test_formatter(self):
        logger = self.check_simple_logger("<accesslog>\n"
                                          "  <syslog>\n"
                                          "    level error\n"
                                          "    facility local3\n"
                                          "    format %(message)s\n"
                                          "  </syslog>\n"
                                          "</accesslog>")
        self.assertEqual(len(logger.handlers), 1)
        syslog = logger.handlers[0]
        self.assertEqual(syslog.level, logging.ERROR)
        self.assertIsInstance(syslog, loghandler.SysLogHandler)
        self.assertEqual(syslog.formatter._fmt, "%(message)s")
        syslog.close()  # avoid ResourceWarnings

    def check_simple_logger(self, text):
        conf = self.get_config(text)
        self.assertIsNotNone(conf.accesslog)
        logger = conf.accesslog()
        self.assertIsInstance(logger, logging.Logger)
        self.assertFalse(logger.propagate)
        self.assertEqual(logger.name, "accesslog")
        self.assertEqual(logger.level, logging.INFO)
        return logger



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Configuration support for the access log.

This assumes that access logging is being performed through the logger
object returned by logging.getLogger('accesslog').
"""
import logging

from ZConfig.components.logger.logger import LoggerFactoryBase


class AccessLogFactory(LoggerFactoryBase):
    """Logger factory that returns the access logger."""

    name = "accesslog"

    def create(self):
        logger = LoggerFactoryBase.create(self)
        logger.setLevel(logging.INFO)
        logger.propagate = False
        formatter = logging.Formatter()
        for handler in logger.handlers:
            handler.setFormatter(formatter)
        return logger



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Implementation of the mkzopeinstance script.

This creates a new instances of the Zope server instance home.  An
'instance home' contains two things:

- application server configuration and data

- server process control scripts and data
"""


import optparse
import os
import shutil
import sys
from xml.sax.saxutils import quoteattr as xml_quoteattr

from zope.app.applicationcontrol import zopeversion
from zope.password import password

import zope.app.server


def main(argv=None, from_checkout=False):
    """Top-level script function to create a new Zope instance."""
    if argv is None:
        argv = sys.argv
    try:
        options = parse_args(argv, from_checkout)
    except SystemExit as e:
        if e.code:
            return 2
        else:
            return 0
    app = Application(options)
    try:
        return app.process()
    except KeyboardInterrupt:
        return 1
    except SystemExit as e:
        return e.code


class Application:

    def __init__(self, options):
        self.options = options
        self.need_blank_line = False

    def read_input_line(self, prompt):
        # The tests replace this to make sure the right things happen.
        if not self.options.interactive:
            print('Error: Tried to ask for user input in'
                  ' non-interactive mode.', file=sys.stderr)
            sys.exit(1)
        return input(prompt)

    def read_password(self, prompt):
        # The tests replace this to make sure the right things happen.
        if not self.options.interactive:
            print('Error: Tried to ask for user input in'
                  ' non-interactive mode.', file=sys.stderr)
            sys.exit(1)
        import getpass
        try:
            return getpass.getpass(prompt)
        except KeyboardInterrupt:
            # The cursor was left on the same line as the prompt,
            # which we don't like.  Print a blank line.
            print()
            raise

    def process(self):
        options = self.options

        # make sure we can find the skeleton
        if not os.path.isdir(options.skeleton):
            print("skeleton directory", options.skeleton, file=sys.stderr)
            print("does not exist or is not a directory", file=sys.stderr)
            return 1

        # create the destination
        if not options.destination:
            options.destination = self.get_skeltarget()
        options.destination = os.path.abspath(options.destination)
        if not os.path.exists(options.destination):
            try:
                os.mkdir(options.destination)
            except OSError as e:
                print("could not create instance home:", e, file=sys.stderr)
                return 1
        elif not os.path.isdir(options.destination):
            print(options.destination, "is not a directory", file=sys.stderr)
            print("(instance homes cannot be created in non-directories)",
                  file=sys.stderr)
            return 1

        if not options.username:
            options.username = self.get_username()

        (options.password_manager,
            password_manager) = self.get_password_manager()

        if not options.password:
            options.password = self.get_password()

        options.password = password_manager.encodePassword(options.password)
        if not isinstance(options.password, str):
            options.password = options.password.decode()

        # now create the instance!
        self.copy_skeleton()
        if options.add_package_includes:
            # need to copy ZCML differently since it's not in the skeleton:
            import __main__
            swhome = os.path.dirname(
                os.path.dirname(os.path.realpath(__main__.__file__)))
            shutil.copy2(os.path.join(swhome, "securitypolicy.zcml"),
                         os.path.join(options.destination, "etc"))
        return 0

    def get_skeltarget(self):
        self.print_message(SKELTARGET_MESSAGE)
        self.need_blank_line = True
        while True:
            skeltarget = self.read_input_line("Directory: ").strip()
            if skeltarget == '':
                print('You must specify a directory', file=sys.stderr)
                continue
            return os.path.expanduser(skeltarget)

    def get_username(self):
        self.print_message(USERNAME_MESSAGE)
        self.need_blank_line = True
        while True:
            username = self.read_input_line("Username: ").strip()
            if not username:
                print(
                    "You must specify an administrative user",
                    file=sys.stderr)
                continue
            return username

    def get_password(self):
        self.print_message(PASSWORD_MESSAGE)
        while True:
            password = self.read_password("Password: ")
            if not password:
                print("Password may not be empty", file=sys.stderr)
                continue
            if password != password.strip() or password.split() != [password]:
                print("Password may not contain spaces", file=sys.stderr)
                continue
            break
        again = self.read_password("Verify password: ")
        if again != password:
            print("Password not verified!", file=sys.stderr)
            sys.exit(1)
        return password

    def get_password_manager(self):
        if not self.options.password_manager and not self.options.interactive:
            self.options.password_manager = password.managers[0][0]

        if self.options.password_manager:
            for name, manager in password.managers:
                if name == self.options.password_manager:
                    return (name, manager)
            print("Unknown password manager!", file=sys.stderr)
            sys.exit(1)

        self.print_message(PASSWORD_MANAGER_MESSAGE)
        for i, (name, manager) in enumerate(password.managers):
            print("% i. %s" % (i + 1, name))
        print()
        self.need_blank_line = True
        while True:
            password_manager = self.read_input_line(
                "Password Manager Number [1]: ")
            if not password_manager:
                index = 0
                break
            elif password_manager.isdigit():
                index = int(password_manager)
                if index > 0 and index <= len(password.managers):
                    index -= 1
                    break
            print("You must select a password manager", file=sys.stderr)
        print("%r password manager selected" % password.managers[index][0])
        return password.managers[index]

    def print_message(self, message):
        if self.need_blank_line:
            print()
            self.need_blank_line = False
        print(message)

    def copy_skeleton(self):
        options = self.options
        # TODO we should be able to compute the script
        script = os.path.abspath(sys.argv[0])
        zope_home = os.path.dirname(os.path.dirname(script))
        zope_init = os.path.dirname(os.path.dirname(
            os.path.abspath(zope.app.server.__file__)))
        software_home = os.path.dirname(os.path.dirname(zope_init))
        self.replacements = [
            ("<<USERNAME>>", options.username),
            ("<<USERNAME-XMLATTR>>", xml_quoteattr(options.username)),
            ("<<PASSWORD>>", options.password),
            ("<<PASSWORD-XMLATTR>>", xml_quoteattr(options.password)),
            ("<<PASSWORD_MANAGER>>", options.password_manager),
            ("<<PYTHON>>", sys.executable),
            ("<<INSTANCE_HOME>>", options.destination),
            ("<<ZOPE_HOME>>", zope_home),
            ("<<SOFTWARE_HOME>>", software_home),
        ]
        self.copytree(self.options.skeleton, self.options.destination)
        if options.zserver:
            self.copytree(
                os.path.join(os.path.dirname(zope.app.server.__file__),
                             'zopeskel'),
                self.options.destination,
            )

    def copytree(self, src, dst):
        # Similar to shutil.copytree(), but doesn't care about
        # symlinks, doesn't collect errors, and uses self.copyfile()
        # instead of shutil.copy2().
        assert os.path.isdir(dst), dst
        names = os.listdir(src)
        if ".svn" in names:
            names.remove(".svn")
        for name in names:
            srcname = os.path.join(src, name)
            dstname = os.path.join(dst, name)
            if os.path.isdir(srcname):
                if not os.path.exists(dstname):
                    os.mkdir(dstname)
                self.copytree(srcname, dstname)
            else:
                self.copyfile(srcname, dstname)
            # There shouldn't be any need to deal with devices, sockets etc.

    def copyfile(self, src, dst):
        if dst.endswith(".in"):
            dst = dst[:-3]
            text = open(src).read()
            # perform replacements
            for var, string in self.replacements:
                text = text.replace(var, string)

            # If the file exists, keep the old file.  This is a
            # hopefully temporary hack to get around distutils
            # stripping the permissions on the server skeletin files.
            # We reuse the original default files, which have the
            # right permissions.
            old = os.path.exists(dst)
            if old:
                f = open(dst, "r+")
                f.truncate(0)
            else:
                f = open(dst, "w")

            f.write(text)
            f.close()

            if not old:
                shutil.copymode(src, dst)
                shutil.copystat(src, dst)
        else:
            shutil.copy2(src, dst)


SKELTARGET_MESSAGE = """\
Please choose a directory in which you'd like to install Zope
'instance home' files such as database files, configuration files,
etc.
"""

USERNAME_MESSAGE = """\
Please choose a username for the initial administrator account.
This is required to allow Zope's management interface to be used.
"""

PASSWORD_MESSAGE = """\
Please provide a password for the initial administrator account.
"""

PASSWORD_MANAGER_MESSAGE = """\
Please select a password manager which will be used for encode the password of
the initial administrator account.
"""


def parse_args(argv, from_checkout=False):
    """Parse the command line, returning an object representing the input."""
    path, prog = os.path.split(os.path.realpath(argv[0]))
    version = "%prog for " + zopeversion.ZopeVersionUtility.getZopeVersion()
    p = optparse.OptionParser(prog=prog,
                              usage="%prog [options]",
                              version=version)
    p.add_option("-d", "--dir", dest="destination", metavar="DIR",
                 help="the dir in which the instance home should be created")
    p.add_option("-s", "--skelsrc", dest="skeleton", metavar="DIR",
                 help="template skeleton directory")
    p.add_option("-m", "--password-manager",
                 dest="password_manager", metavar="NAME",
                 help=("set the name of the password manager"
                       " to be used for encode the password"))
    p.add_option("-u", "--user", dest="username", metavar="USER:PASSWORD",
                 help="set the user name and password of the initial user")
    p.add_option("--non-interactive", dest="interactive", action="store_false",
                 default=True, help="do no interactive prompting")
    p.add_option("--zserver", dest="zserver", action="store_true",
                 help="""\
Use the older ZServer network server rather than the default Twisted
server.  The Twisted integration with Zope is experimental and not
recommended for use in production sites.  We do encourage it's
use in development or in sites that can stand a little uncertianty, so
that we can gain more experience with it.
""",
                 )

    options, args = p.parse_args(argv[1:])
    if options.skeleton is None:
        options.add_package_includes = from_checkout
        basedir = os.path.dirname(path)
        # no assurance that this exists!
        options.skeleton = os.path.join(basedir, "zopeskel")
    else:
        options.add_package_includes = False
    options.program = prog
    options.version = version
    if args:
        p.error("too many arguments")
    options.password = None
    if options.username and ":" in options.username:
        options.username, options.password = options.username.split(":", 1)
    return options



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Functions that control how the Zope appserver knits itself together.
"""
import asyncore
import logging
import os
import sys
from time import process_time
from time import time as wall_clock_time

import zope.app.appsetup.appsetup
import zope.app.appsetup.product
import zope.processlifetime
from zdaemon import zdoptions
from zope.event import notify
from zope.server.taskthreads import ThreadedTaskDispatcher


CONFIG_FILENAME = "zope.conf"


class ZopeOptions(zdoptions.ZDOptions):

    logsectionname = None

    def default_configfile(self):
        # XXX: this probably assumes a monolithic zope 3 source tree
        # layout and isn't likely to work
        dir = os.path.normpath(
            os.path.join(os.path.dirname(__file__),
                         os.pardir, os.pardir, os.pardir, os.pardir))
        for filename in [CONFIG_FILENAME, CONFIG_FILENAME + ".in"]:
            filename = os.path.join(dir, filename)
            if os.path.isfile(filename):
                return filename
        return None


exit_status = None


def main(args=None):
    # Record start times (real time and CPU time)
    t0 = wall_clock_time()
    c0 = process_time()

    setup(load_options(args))

    t1 = wall_clock_time()
    c1 = process_time()
    logging.info("Startup time: %.3f sec real, %.3f sec CPU", t1 - t0, c1 - c0)

    run()
    sys.exit(exit_status or 0)


def debug(args=None):
    options = load_options(args)

    zope.app.appsetup.product.setProductConfigurations(
        options.product_config)

    zope.app.appsetup.config(options.site_definition)

    db = zope.app.appsetup.appsetup.multi_database(options.databases)[0][0]
    notify(zope.processlifetime.DatabaseOpened(db))
    return db


def run():
    try:
        global exit_status
        while asyncore.socket_map and exit_status is None:
            asyncore.poll(30.0)
    except KeyboardInterrupt:
        # Exit without spewing an exception.
        pass


def load_options(args=None):
    if args is None:
        args = sys.argv[1:]
    options = ZopeOptions()
    options.schemadir = os.path.dirname(os.path.abspath(__file__))
    options.realize(args)
    options = options.configroot

    if options and options.path:
        sys.path[:0] = [os.path.abspath(p) for p in options.path]
    return options


def setup(options):
    sys.setcheckinterval(options.check_interval)

    zope.app.appsetup.product.setProductConfigurations(
        options.product_config)
    options.eventlog()
    options.accesslog()
    for logger in options.loggers:
        logger()

    features = ('zserver',)
    # Provide the devmode, if activated
    if options.devmode:
        features += ('devmode',)
        logging.warning(
            "Developer mode is enabled: this is a security risk "
            "and should NOT be enabled on production servers. Developer mode "
            "can be turned off in etc/zope.conf")

    zope.app.appsetup.config(options.site_definition, features=features)

    db = zope.app.appsetup.appsetup.multi_database(options.databases)[0][0]

    notify(zope.processlifetime.DatabaseOpened(db))

    task_dispatcher = ThreadedTaskDispatcher()
    task_dispatcher.setThreadCount(options.threads)

    for server in options.servers:
        server.create(task_dispatcher, db)

    notify(zope.processlifetime.ProcessStarting())

    return db



##############################################################################
#
# Copyright (c) 2001,2002,2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Server Control Implementation
"""

from zope.app.applicationcontrol.interfaces import IServerControl
from zope.interface import implementer

import zope.app.server.main


@implementer(IServerControl)
class ServerControl:

    def shutdown(self, time=0):
        """See zope.app.applicationcontrol.interfaces.IServerControl"""
        # TODO: Graceful shutdown does not work yet.

        # This will work for servers started directly and by zdaemon. Passing
        # an exit status of 0 causes zdaemon to not restart the process.
        zope.app.server.main.exit_status = 0

    def restart(self, time=0):
        """See zope.app.applicationcontrol.interfaces.IServerControl"""
        # TODO: Graceful restart does not work yet.

        # TODO: Make sure this is only called if we are running via zdaemon.

        # Passing an exit status of 1 causes zdaemon to restart the process.
        zope.app.server.main.exit_status = 1


serverControl = ServerControl()



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for zope.app.server.servertype
"""
import doctest

from zope.interface.verify import verifyObject


def doctest_ServerType():
    r"""Tests for ServerType

    Zope 3 has many server types -- HTTP, FTP, HTTP with postmortem debugging,
    etc.  All of them are registered as IServerType utilities in ZCML.

    ServerType is an implementation of IServerType.  The constructor of
    ServerType takes quite a few arguments, a number of which are factories.
    We will use stubs of those.

    The 'factory' argument specifies the server factory (e.g.
    PublisherHTTPServer from zope.server.http.httpserver)

        >>> def factory(request_factory, name, ip, port,
        ...             task_dispatcher=None, hit_log=None, verbose=False):
        ...     if ip == '':
        ...         ip = '*' # listen on all network interfaces
        ...     print("Starting a server (%s) on %s:%d" % (name, ip, port))
        ...     print("This server will use %s to construct requests" %
        ...           request_factory)
        ...     print("This server will use %s for hit logging" % hit_log)
        ...     if verbose:
        ...         print("This server will be verbose")
        ...     else:
        ...         print("This server will not be verbose")
        ...     print("This server will be managed by %s" % task_dispatcher)

    The 'requestFactory' argument specifies a function that returns a factory
    for requests (e.g.  HTTPPublicationRequestFactory from
    zope.server.http.publisherhttpserver).  It is, in fact, a request factory
    factory.

        >>> def requestFactory(db):
        ...     return 'my request factory for %s' % db

    The 'logFactory' argument specifies the factory for an access logger (e.g.
    CommonAccessLogger from zope.server.http.commonaccesslogger).

        >>> def logFactory():
        ...     return 'my logger'

    The 'defaultPort' argument specifies the default TCP port number for the
    server.

    The 'defaultVerbose' argument specifies the default verbosity.

    The 'defaultIP' argument specifies the network interface for listening on.
    You can specify the network interface IP address, or an empty string if you
    want to listen on all interfaces.

        >>> from zope.app.server.servertype import IServerType
        >>> from zope.app.server.servertype import ServerType
        >>> st = ServerType(factory, requestFactory, logFactory,
        ...                 defaultPort=8080, defaultVerbose=False)
        >>> verifyObject(IServerType, st)
        True

    A server type is then registered as a named utility.  These utilities are
    used while interpreting <server> sections of zope.conf to create instances
    of servers listening on a specific port.

    When you create an instance of a server, you need to tell it the task
    dispatcher (see IDispatcher in zope.server.interfaces), and the ZODB
    database object.

    The `name` argument to create is, as far as I can tell, purely informative.
    It is used to construct a server identifier that appears in log files and,
    for example, the 'Server' HTTP header.

        >>> dispatcher = 'my task dispatcher'
        >>> db = 'my database'
        >>> st.create('Sample Server', dispatcher, db)
        Starting a server (Sample Server) on *:8080
        This server will use my request factory for my database to construct requests
        This server will use my logger for hit logging
        This server will not be verbose
        This server will be managed by my task dispatcher

    You can, of course, create multiple instances of the same server type, and
    bind them to different ports.

        >>> st.create('Sample Server 2', dispatcher, db, port=1234, verbose=True)
        Starting a server (Sample Server 2) on *:1234
        This server will use my request factory for my database to construct requests
        This server will use my logger for hit logging
        This server will be verbose
        This server will be managed by my task dispatcher

        >>> st.create('Sample Server 3', dispatcher, db, port=9090,
        ...           ip='127.0.0.1')
        Starting a server (Sample Server 3) on 127.0.0.1:9090
        This server will use my request factory for my database to construct requests
        This server will use my logger for hit logging
        This server will not be verbose
        This server will be managed by my task dispatcher

    """  # noqa: E501 line too long


def test_suite():
    return doctest.DocTestSuite()



import unittest

from zope.applicationcontrol.interfaces import IServerControl
from zope.interface.verify import verifyObject

import zope.app.server.main
from zope.app.server.servercontrol import serverControl


class TestServerControl(unittest.TestCase):

    def tearDown(self):
        zope.app.server.main.exit_status = None

    def test(self):
        verifyObject(IServerControl, serverControl)

    def test_shutdown(self):
        serverControl.shutdown()
        self.assertEqual(zope.app.server.main.exit_status, 0)

    def test_restart(self):
        serverControl.restart()
        self.assertEqual(zope.app.server.main.exit_status, 1)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Server Type
"""
from zope.interface import Interface
from zope.interface import implementer


class IServerType(Interface):
    """Server type utility.

    This is a pure read-only interface, since the values are set through
    a ZCML directive and we shouldn't be able to change them.
    """

    def create(name, task_dispatcher, db, port=None, verbose=None, ip=None):
        """Create the server knowing the port, task dispatcher and the ZODB.

        Returns the new server.
        """


@implementer(IServerType)
class ServerType:

    def __init__(self, factory, requestFactory, logFactory,
                 defaultPort, defaultVerbose, defaultIP=''):
        self._factory = factory
        self._requestFactory = requestFactory
        self._logFactory = logFactory
        self._defaultPort = defaultPort
        self._defaultVerbose = defaultVerbose
        self._defaultIP = defaultIP

    def create(self, name, task_dispatcher, db, port=None,
               verbose=None, ip=None):
        'See IServerType'

        request_factory = self._requestFactory(db)

        if port is None:
            port = self._defaultPort

        if ip is None:
            ip = self._defaultIP

        if verbose is None:
            verbose = self._defaultVerbose

        return self._factory(request_factory, name, ip, port,
                             task_dispatcher=task_dispatcher,
                             verbose=verbose,
                             hit_log=self._logFactory(),
                             )



import unittest


class TestZpasswdDeprecation(unittest.TestCase):

    def test(self):
        with self.assertWarns(DeprecationWarning):
            from zope.app.server.zpasswd import Principal  # noqa



import unittest

from zope.app.server.main import load_options
from zope.app.server.main import main
from zope.app.server.tests import capture_output


class TestMain(unittest.TestCase):

    def test_main(self):
        with self.assertRaises(SystemExit) as cm:
            with capture_output() as (stdout, stderr):
                main(['invalid'])
        self.assertEqual(cm.exception.code, 2)
        self.assertNotEqual(stderr.getvalue(), "")

    def test_load_options_no_config_file_specified(self):
        options = load_options([])
        self.assertIs(options, None)

    # TODO: somebody finish this please



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""FTP server
"""
import zope.interface
from zope.app.publication.ftp import FTPPublication
from zope.app.publication.interfaces import IPublicationRequestFactory
from zope.publisher.ftp import FTPRequest
from zope.server.ftp.logger import CommonFTPActivityLogger
from zope.server.ftp.publisher import PublisherFTPServer

from zope.app.server.servertype import ServerType


@zope.interface.implementer(IPublicationRequestFactory)
class FTPRequestFactory:
    """FTP Request factory

    FTP request factories for a given database create FTP requests with
    publications on the given database:

    .. The test below has been disabled and moved to test_ftp.py (LP #257954)

      >>> from ZODB.tests.util import DB
      >>> db = DB()
      >>> factory = FTPRequestFactory(db)
      >>> from io import BytesIO
      >>> request = factory(BytesIO(b''), {'credentials': None, 'path': '/'})
      >>> request.publication.db is db
      True
      >>> db.close()

    """

    def __init__(self, db):
        self.publication = FTPPublication(db)

    def __call__(self, input_stream, env):
        request = FTPRequest(input_stream, env)
        request.setPublication(self.publication)
        return request


server = ServerType(
    PublisherFTPServer,
    FTPRequestFactory,
    CommonFTPActivityLogger,
    8021, True)



import unittest

from zope.interface.verify import verifyObject
from zope.server.http.wsgihttpserver import WSGIHTTPServer

from zope.app.server.servertype import IServerType
from zope.app.server.wsgi import http
from zope.app.server.wsgi import pmhttp


class TestWSGIServerType(unittest.TestCase):

    def test(self):
        verifyObject(IServerType, http)
        verifyObject(IServerType, pmhttp)

    def test_create(self):
        dispatcher = 'my task dispatcher'
        db = 'my database'
        server = http.create("HTTP", dispatcher, db, port=0)
        self.assertIsInstance(server, WSGIHTTPServer)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Doc tests for the FTP server.
"""

import unittest
from io import BytesIO


class Tests(unittest.TestCase):

    def test_ftp(self):
        from ZODB.tests.util import DB

        from zope.app.server.ftp import FTPRequestFactory
        db = DB()
        factory = FTPRequestFactory(db)
        request = factory(BytesIO(b''), {'credentials': None, 'path': '/'})
        self.assertTrue(request.publication.db is db)
        db.close()



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""WSGI-compliant HTTP server setup.
"""

import zope.interface
from zope.app.publication.httpfactory import HTTPPublicationRequestFactory
from zope.app.wsgi import WSGIPublisherApplication
from zope.server.http import wsgihttpserver
from zope.server.http.commonaccesslogger import CommonAccessLogger

from zope.app.server import servertype


@zope.interface.implementer(servertype.IServerType)
class ServerType:

    def __init__(self, factory, applicationFactory, logFactory,
                 defaultPort, defaultVerbose, defaultIP='',
                 requestFactory=HTTPPublicationRequestFactory):
        self._factory = factory
        self._applicationFactory = applicationFactory
        self._requestFactory = requestFactory
        self._logFactory = logFactory
        self._defaultPort = defaultPort
        self._defaultVerbose = defaultVerbose
        self._defaultIP = defaultIP

    def create(self, name, task_dispatcher, db, port=None,
               verbose=None, ip=None):
        'See IServerType'

        application = self._applicationFactory(
            db, factory=self._requestFactory)

        if port is None:
            port = self._defaultPort

        if ip is None:
            ip = self._defaultIP

        if verbose is None:
            verbose = self._defaultVerbose

        return self._factory(application, name, ip, port,
                             task_dispatcher=task_dispatcher,
                             verbose=verbose,
                             hit_log=self._logFactory(),
                             )


http = ServerType(wsgihttpserver.WSGIHTTPServer,
                  WSGIPublisherApplication,
                  CommonAccessLogger,
                  8080, True)

pmhttp = ServerType(wsgihttpserver.PMDBWSGIHTTPServer,
                    WSGIPublisherApplication,
                    CommonAccessLogger,
                    8013, True)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for the implementation of the mkzopeinstance script.
"""

import os
import shutil
import tempfile
import unittest

from zope.app.server import mkzopeinstance
from zope.app.server.tests import capture_output


class ArgumentParsingTestCase(unittest.TestCase):
    """Ensure the command line is properly converted to an options
    object.
    """

    def parse_args(self, args, stdout=None, stderr=None):
        argv = ["foo/bar.py"] + args
        with capture_output(stdout, stderr):
            options = mkzopeinstance.parse_args(argv)
        self.assertEqual(options.program, "bar.py")
        self.assertTrue(options.version)
        return options

    def test_no_arguments(self):
        self.parse_args([])

    def test_version_long(self):
        self.check_stdout_content(["--version"])

    def test_help_long(self):
        self.check_stdout_content(["--help"])

    def test_help_short(self):
        self.check_stdout_content(["-h"])

    def check_stdout_content(self, args):
        with self.assertRaises(SystemExit) as cm:
            with capture_output() as (stdout, stderr):
                self.parse_args(args, stdout, stderr)
        self.assertEqual(cm.exception.code, 0)
        self.assertNotEqual(stdout.getvalue(), "")
        self.assertEqual(stderr.getvalue(), "")

    def test_without_destination(self):
        options = self.parse_args([])
        self.assertEqual(options.destination, None)

    def test_destination_long(self):
        options = self.parse_args(["--dir", "some/dir"])
        self.assertEqual(options.destination, "some/dir")

    def test_destination_short(self):
        options = self.parse_args(["-d", "some/dir"])
        self.assertEqual(options.destination, "some/dir")

    def test_without_skeleton(self):
        # make sure we get *some* skeleton directory by default
        # there's no claim that it exists
        options = self.parse_args([])
        self.assertNotEqual(options.skeleton, None)

    def test_with_skeleton_long(self):
        options = self.parse_args(["--skelsrc", "some/dir"])
        self.assertEqual(options.skeleton, "some/dir")
        self.assertFalse(options.add_package_includes)

    def test_with_skeleton_short(self):
        options = self.parse_args(["-s", "some/dir"])
        self.assertEqual(options.skeleton, "some/dir")
        self.assertFalse(options.add_package_includes)

    def test_without_username(self):
        options = self.parse_args([])
        self.assertEqual(options.username, None)
        self.assertEqual(options.password, None)

    def test_username_without_password_long(self):
        options = self.parse_args(["--user", "User"])
        self.assertEqual(options.username, "User")
        self.assertEqual(options.password, None)

    def test_username_without_password_short(self):
        options = self.parse_args(["-u", "User"])
        self.assertEqual(options.username, "User")
        self.assertEqual(options.password, None)

    def test_username_with_password_long(self):
        options = self.parse_args(["--user", "User:Pass"])
        self.assertEqual(options.username, "User")
        self.assertEqual(options.password, "Pass")

    def test_username_with_password_short(self):
        options = self.parse_args(["-u", "User:Pass"])
        self.assertEqual(options.username, "User")
        self.assertEqual(options.password, "Pass")

    def test_without_password_manager(self):
        options = self.parse_args([])
        self.assertEqual(options.password_manager, None)

    def test_password_manager_short(self):
        options = self.parse_args(["-m", "Manager"])
        self.assertEqual(options.password_manager, "Manager")

    def test_password_manager_long(self):
        options = self.parse_args(["--password-manager", "Manager"])
        self.assertEqual(options.password_manager, "Manager")

    def test_junk_positional_arg(self):
        with self.assertRaises(SystemExit) as cm:
            self.parse_args(["junk"])
        self.assertNotEqual(cm.exception.code, 0)


class InputCollectionTestCase(unittest.TestCase):

    def setUp(self):
        super().setUp()
        self.tmpdir = tempfile.mkdtemp(prefix="test-mkzopeinstance-")
        self.skeleton = os.path.join(self.tmpdir, "skel")
        self.instance = os.path.join(self.tmpdir, "inst")
        os.mkdir(self.skeleton)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)
        super().tearDown()

    def createOptions(self):
        options = Options()
        options.skeleton = self.skeleton
        options.interactive = True
        return options

    def test_get_skeltarget(self):
        options = self.createOptions()
        input = ["  ", " foo "]
        app = ControlledInputApplication(options, input)
        with capture_output() as (stdout, stderr):
            skel = app.get_skeltarget()
        self.assertEqual(skel, "foo")
        self.assertEqual(input, [])
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_process_creates_destination(self):
        options = self.createOptions()
        input = [self.instance]
        app = ControlledInputApplication(options, input)
        with capture_output() as (stdout, stderr):
            self.assertEqual(app.process(), 0)
        self.assertTrue(os.path.isdir(self.instance))
        self.assertEqual(input, [])
        self.assertTrue(app.all_input_consumed())

    def test_zserver_support(self):
        # test the zserver option.  We should get zserver versions of
        # runzope, zopectl, debugzope and zope.conf.  Any of these
        # that we provide in put skeleton should be overritten.

        # privide a dummy runzope
        os.mkdir(os.path.join(self.skeleton, 'bin'))
        f = open(os.path.join(self.skeleton, 'bin', 'runzope.in'), 'w')
        f.write('runzope')
        f.close()

        options = self.createOptions()
        options.destination = self.instance
        options.interactive = False
        options.zserver = True
        app = ControlledInputApplication(options, [])
        with capture_output() as (stdout, stderr):
            self.assertEqual(app.process(), 0)
        with open(os.path.join(self.instance, 'bin', 'runzope')) as f:
            self.assertIn(
                'from zope.app.server.main import main', f.read())
        with open(os.path.join(self.instance, 'bin', 'debugzope')) as f:
            self.assertIn(
                'from zope.app.server.main import debug', f.read())
        self.assertTrue(os.path.exists(
            os.path.join(self.instance, 'etc', 'zope.conf')
        ))

    def test_process_aborts_on_file_destination(self):
        options = self.createOptions()
        options.destination = self.instance
        open(self.instance, "w").close()
        app = ControlledInputApplication(options, [])
        with capture_output() as (stdout, stderr):
            self.assertEqual(app.process(), 1)
        self.assertTrue(stderr.getvalue())

    def test_process_aborts_on_failed_destination_creation(self):
        options = self.createOptions()
        options.destination = os.path.join(self.instance, "foo")
        app = ControlledInputApplication(options, [])
        with capture_output() as (stdout, stderr):
            self.assertEqual(app.process(), 1)
        self.assertTrue(stderr.getvalue())

    def test_get_username(self):
        options = self.createOptions()
        app = ControlledInputApplication(options, ["myuser"])
        with capture_output() as (stdout, stderr):
            usr = app.get_username()
        self.assertEqual(usr, "myuser")
        self.assertFalse(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_username_strips_whitespace(self):
        options = self.createOptions()
        app = ControlledInputApplication(options, ["  myuser\t"])
        with capture_output() as (stdout, stderr):
            usr = app.get_username()
        self.assertEqual(usr, "myuser")
        self.assertFalse(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_username_ignores_empty_names(self):
        options = self.createOptions()
        app = ControlledInputApplication(options, ["", "  ", "\t", "myuser"])
        with capture_output() as (stdout, stderr):
            usr = app.get_username()
        self.assertEqual(usr, "myuser")
        self.assertTrue(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_password_manager(self):
        options = self.createOptions()
        options.password_manager = None
        app = ControlledInputApplication(options, ["1"])
        with capture_output() as (stdout, stderr):
            name, pwm = app.get_password_manager()
        self.assertEqual(name, "Plain Text")
        self.assertEqual(pwm.encodePassword("foo").decode(), "foo")
        self.assertFalse(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_wrong_password_manager(self):
        options = self.createOptions()
        options.password_manager = "Unknown"
        app = ControlledInputApplication(options, [])
        with self.assertRaises(SystemExit) as cm:
            with capture_output() as (stdout, stderr):
                app.get_password_manager()
        self.assertEqual(cm.exception.code, 1)
        self.assertTrue(stderr.getvalue())
        self.assertFalse(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_password(self):
        options = self.createOptions()
        app = ControlledInputApplication(options, ["foo", "foo"])
        with capture_output() as (stdout, stderr):
            pw = app.get_password()
        self.assertEqual(pw, "foo")
        self.assertFalse(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_password_not_verified(self):
        options = self.createOptions()
        app = ControlledInputApplication(options, ["foo", "bar"])
        with self.assertRaises(SystemExit) as cm:
            with capture_output() as (stdout, stderr):
                app.get_password()
        self.assertEqual(cm.exception.code, 1)
        self.assertTrue(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_password_empty(self):
        # Make sure the empty password is ignored.
        options = self.createOptions()
        app = ControlledInputApplication(options, ["", "foo", "foo"])
        with capture_output() as (stdout, stderr):
            pw = app.get_password()
        self.assertEqual(pw, "foo")
        self.assertTrue(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_get_password_disallows_whitespace(self):
        # Any password that contains spaces is disallowed.
        options = self.createOptions()
        app = ControlledInputApplication(options, [" ", "\t", "a b",
                                                   " a", "b ", "foo", "foo"])
        with capture_output() as (stdout, stderr):
            pw = app.get_password()
        self.assertEqual(pw, "foo")
        self.assertTrue(stderr.getvalue())
        self.assertTrue(stdout.getvalue())
        self.assertTrue(app.all_input_consumed())

    def test_can_rewrite_existing_instance(self):
        # Fill out the skeleton a little so we test more cases:
        os.mkdir(os.path.join(self.skeleton, "etc"))
        f = open(os.path.join(self.skeleton, "etc", "README.txt"), "w")
        f.write("Configuration goes here.\n")
        f.close()

        # Create an instance home:
        options = self.createOptions()
        options.destination = self.instance
        app = ControlledInputApplication(options, [])
        with capture_output() as (stdout, stderr):
            rc = app.process()
        self.assertEqual(rc, 0)
        self.assertTrue(app.all_input_consumed())
        self.assertTrue(os.path.exists(os.path.join(self.instance, "etc")))

        # Make sure we can do it again:
        options = self.createOptions()
        options.destination = self.instance
        app = ControlledInputApplication(options, [])
        with capture_output() as (stdout, stderr):
            rc = app.process()
        self.assertEqual(rc, 0)
        self.assertTrue(app.all_input_consumed())
        self.assertTrue(os.path.exists(os.path.join(self.instance, "etc")))

    def test_zope_namespace_package_doesnt_affect_software_home(self):
        # Make sure that a zope namespace package in a different
        # location won't affect SOFTWARE_HOME

        # let's mess with zope's __file__
        import zope
        old_path = getattr(zope, '__file__', None)
        zope.__file__ = os.path.join(
            *'and now for something completely different'.split())

        # place a test file into the skeleton dir that'll be expanded
        # to SOFTWARE_HOME by mkzopeinstance
        with open(os.path.join(self.skeleton, 'test.in'), 'w') as f:
            f.write('<<SOFTWARE_HOME>>')

        # run mkzopeinstance
        options = self.createOptions()
        options.destination = self.instance
        app = ControlledInputApplication(options, [])
        with capture_output() as (stdout, stderr):
            app.process()

        # check for the expected output: mkzopeinstance should take
        # zope.app.server as an anchor for determining SOFTWARE_HOME
        import zope.app.server
        expected = os.path.dirname(os.path.dirname(os.path.dirname(
            os.path.dirname(zope.app.server.__file__))))
        with open(os.path.join(self.instance, 'test')) as f:
            self.assertEqual(f.read(), expected)

        # cleanup the fake 'zope' module
        if old_path is None:
            del zope.__file__
        else:  # pragma: no cover
            zope.__file__ = old_path


class ControlledInputApplication(mkzopeinstance.Application):

    def __init__(self, options, input_lines):
        mkzopeinstance.Application.__init__(self, options)
        self.__input = input_lines

    def read_input_line(self, prompt):
        return self.__input.pop(0)

    read_password = read_input_line

    def all_input_consumed(self):
        return not self.__input


class Options:

    username = "[test-username]"
    password_manager = "Plain Text"
    password = "[test-password]"
    destination = None
    version = "[test-version]"
    program = "[test-program]"
    add_package_includes = False
    zserver = False



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# https://zopetoolkit.readthedocs.io/
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.app.server package
"""
import os

from setuptools import find_packages
from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


setup(
    name='zope.app.server',
    version='5.0',
    author='Zope Corporation and Contributors',
    author_email='zope-dev@zope.dev',
    description='ZServer integration for Zope 3 Applications',
    long_description=(
        read('README.rst')
        + '\n\n' +
        read('CHANGES.rst')
    ),
    keywords="zope3 zserver server http ftp wsgi",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope :: 3',
    ],
    url='https://github.com/zopefoundation/zope.app.server',
    license='ZPL 2.1',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['zope', 'zope.app'],
    python_requires='>=3.7',
    extras_require=dict(
        test=[
            'zope.testrunner',
            'zope.app.testing',
            'ZODB[test]',
        ],
    ),
    install_requires=[
        'setuptools',
        'zope.app.applicationcontrol',
        'zope.app.appsetup >= 3.11',
        'zope.app.publication',
        'zope.app.wsgi',
        'zope.configuration',
        'zope.deprecation',
        'zope.event',
        'zope.interface',
        'zope.publisher',
        'zope.server >= 4.0',
        'zope.password >= 3.6',
        'zope.processlifetime',
        'zdaemon',
        'ZConfig',
    ],
    include_package_data=True,
    zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for zope.app.server.server
"""
import doctest

from zope.app.testing import setup
from zope.component import provideUtility


def doctest_ServerFactory():
    r"""Tests for ServerFactory

    Zope 3 has many server types -- HTTP, FTP, HTTP with postmortem debugging,
    etc.  All of them are registered as IServerType utilities in ZCML.

        >>> setup.placelessSetUp()

        >>> from zope.interface import implementer
        >>> from zope.app.server.servertype import IServerType
        >>> @implementer(IServerType)
        ... class MyServerType:
        ...     def create(self, name, task_dispatcher, db, port='unknown',
        ...                verbose='unspecified', ip=''):
        ...         if not ip:
        ...             ip = '*' # listen on all interfaces
        ...         return ('%s server on %s:%d, registered with %s,\n'
        ...                 'serving from %s, verbosity %s'
        ...                 % (name, ip, port, task_dispatcher, db, verbose))
        >>> provideUtility(MyServerType(), IServerType, name='HTTP')
        >>> provideUtility(MyServerType(), IServerType, name='FTP')

    ServerFactory is used to hook into ZConfig and create instances of servers
    specified in zope.conf.  It gets a `section` argument that contains
    settings specified in a ZConfig <server> section.

        >>> class ServerSectionStub:
        ...     type = 'HTTP'
        ...     address = ('', 8080)
        ...     verbose = False
        >>> my_section = ServerSectionStub()

        >>> from zope.app.server.server import ServerFactory
        >>> sf = ServerFactory(my_section)

    The server factory object knows how to create a server, given a task
    dispatcher (see IDispatcher from zope.server.interfaces) and a ZODB
    database object.

        >>> task_dispatcher = 'my task dispatcher'
        >>> db = 'my db'
        >>> print(sf.create(task_dispatcher, db))
        HTTP server on *:8080, registered with my task dispatcher,
        serving from my db, verbosity False

    The settings actually work

        >>> my_section.type = 'FTP'
        >>> my_section.address = ('127.0.0.1', 8021)
        >>> my_section.verbose = True
        >>> sf = ServerFactory(my_section)
        >>> print(sf.create(task_dispatcher, db))
        FTP server on 127.0.0.1:8021, registered with my task dispatcher,
        serving from my db, verbosity True

    That's it.

        >>> setup.placelessTearDown()

    """


def test_suite():
    return doctest.DocTestSuite()



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Datatype for a <server> section in a Zope 3 configuration file.

This is called by the ZConfig machinery while processing a configuration.
"""
import zope.component

from zope.app.server.servertype import IServerType


class ServerFactory:
    """Factory for server objects.

    The factories are part of the configuration data returned by
    ZConfig.
    """

    def __init__(self, section):
        """Initialize the factory based on a <server> section."""
        self.type = section.type
        self.address = section.address
        self.verbose = section.verbose

    def create(self, task_dispatcher, database):
        """Return a server based on the server types defined via ZCML."""

        servertype = zope.component.getUtility(IServerType, self.type)
        # The server object self-registers with the asyncore mainloop.
        return servertype.create(
            self.type,
            task_dispatcher, database,
            ip=self.address[0],
            port=self.address[1],
            verbose=self.verbose)



import sys
from contextlib import contextmanager
from io import StringIO


@contextmanager
def capture_output(stdout=None, stderr=None):
    old_stdout = sys.stdout
    old_stderr = sys.stderr
    sys.stdout = stdout = stdout or StringIO()
    sys.stderr = stderr = stderr or StringIO()
    try:
        yield stdout, stderr
    finally:
        sys.stdout = old_stdout
        sys.stderr = old_stderr



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test that the Zope appserver configuration schema can be loaded.
"""
import os.path
import unittest

import ZConfig


class TestConfiguration(unittest.TestCase):

    def test_schema(self):
        dir = os.path.dirname(os.path.dirname(__file__))
        filename = os.path.join(dir, "schema.xml")
        ZConfig.loadSchema(filename)
